@@ -4,6 +4,7 @@
 from __future__ import annotations
 
 from dataclasses import MISSING, field
+from types import MappingProxyType
 from typing import Any
 
 from pytreeclass._src.tree_util import _tree_fields, _tree_structure
@@ -12,12 +13,9 @@ from pytreeclass._src.tree_util import _tree_fields, _tree_structure
 class _treeBase:
     def __new__(cls, *args, **kwargs):
         self = object.__new__(cls)
-        
-        # definition of `__undeclared_fields__` attribute
-        object.__setattr__(self, "__undeclared_fields__", {})
-
+        object.__setattr__(self, "__undeclared_fields__", MappingProxyType({}))
         # set default values to class instance
-        # Note: ideally this method should be called once to avoid multiple
+        # Note: ideally this method should be called once
         for field_item in self.__dataclass_fields__.values():
             if field_item.default is not MISSING:
                 object.__setattr__(self, field_item.name, field_item.default)
@@ -28,15 +26,10 @@ class _treeBase:
         """Flatten rule for `jax.tree_flatten`
 
         Returns:
-            Tuple of dynamic values and (dynamic keys,static dict, cached values)
+            Tuple of dynamic values and (dynamic keys,static dict)
         """
-        if hasattr(self, "__pytree_structure_cache__"):
-            # return the cached pytree_structure
-            return (), ((), (), (self.__pytree_structure_cache__))
-
-        else:
-            dynamic, static = _tree_structure(self)
-            return dynamic.values(), (dynamic.keys(), (static), ())
+        dynamic, static = _tree_structure(self)
+        return dynamic.values(), (dynamic.keys(), static)
 
     @classmethod
     def tree_unflatten(cls, treedef, leaves):
@@ -45,7 +38,7 @@ class _treeBase:
         Args:
             treedef:
                 Pytree definition
-                includes Dynamic nodes keys , static dictionary and frozen state
+                includes Dynamic nodes keys , static dictionary
             leaves:
                 Dynamic nodes values
 
@@ -61,18 +54,9 @@ class _treeBase:
         # this approach creates the attribute once.
         self = object.__new__(cls)
 
-        if len(treedef[2]) == 2:
-            # retrieve the cached structure
-            dynamic, static = treedef[2]
-            # pass the cached structure to the new instance
-            object.__setattr__(self, "__pytree_structure_cache__", (dynamic, static))
-        else:
-            dynamic = dict(zip(treedef[0], leaves))
-            static = treedef[1]
-
         # update the instance values with the retrieved dynamic and static values
-        self.__dict__.update(dynamic)
-        self.__dict__.update(static)
+        self.__dict__.update(dict(zip(treedef[0], leaves)))
+        self.__dict__.update(treedef[1])
 
         return self
 
@@ -103,18 +87,5 @@ class _implicitSetter:
             object.__setattr__(field_value, "type", type(value))
 
             # register it to class
-            self.__undeclared_fields__.update({key: field_value})
-
-
-class _explicitSetter:
-    """Register dataclass fields"""
-
-    __immutable_pytree__ = True
-
-    def __setattr__(self, key, value):
-        if self.__immutable_pytree__:
-            raise ImmutableInstanceError(
-                f"Cannot set {key} = {value}. Use `.at['{key}'].set({value!r})` instead."
-            )
-
-        object.__setattr__(self, key, value)
+            new_fields = {**self.__undeclared_fields__, **{key: field_value}}  # fmt: skip
+            object.__setattr__(self, "__undeclared_fields__", MappingProxyType(new_fields))  # fmt: skip
