@@ -1,10 +1,18 @@
 from __future__ import annotations
 
-from dataclasses import field
+from dataclasses import Field, field
+from typing import Any
 
 import pytreeclass._src as src
 from pytreeclass._src.dispatch import dispatch
-from pytreeclass._src.tree_util import _tree_fields, is_treeclass_frozen
+from pytreeclass._src.tree_util import (
+    _tree_fields,
+    is_frozen_field,
+    is_nondiff_field,
+    is_treeclass,
+    is_treeclass_frozen,
+    is_treeclass_nondiff,
+)
 from pytreeclass.tree_viz.node_pprint import (
     _format_node_diagram,
     _format_node_repr,
@@ -12,11 +20,27 @@ from pytreeclass.tree_viz.node_pprint import (
     _format_width,
 )
 
-# from pytreeclass.tree_viz.tree_export import (
-#     _generate_mermaid_link,
-#     _mermaid_table,
-#     _mermaid_table_row,
-# )
+PyTree = Any
+
+
+def _marker(field_item: Field, node_item: Any, default: str = "") -> str:
+    """return the suitable marker given the field and node item
+
+    Args:
+        field_item (Field): field item of the pytree node
+        node_item (Any): node item
+        default (str, optional): default marker. Defaults to "".
+
+    Returns:
+        str: marker character.
+    """
+    # for now, we only have two markers '*' for non-diff and '#' for frozen
+    if is_nondiff_field(field_item) or is_treeclass_nondiff(node_item):
+        return "*"
+    elif is_frozen_field(field_item) or is_treeclass_frozen(node_item):
+        return "#"
+    else:
+        return default
 
 
 def tree_repr(tree, width: int = 60) -> str:
@@ -26,74 +50,57 @@ def tree_repr(tree, width: int = 60) -> str:
         str: indented tree leaves.
     """
 
-    @dispatch(argnum=1)
-    def recurse_field(field_item, node_item, depth, is_frozen, is_last_field):
+    @dispatch(argnum="node_item")
+    def recurse_field(
+        field_item: Field, node_item: Any, depth: int, is_last_field: bool
+    ):
         """format non-treeclass field"""
         nonlocal FMT
 
         if field_item.repr:
-            is_static = field_item.metadata.get("static", False)
-            mark = "*" if is_static else ("#" if is_frozen else "")
-
+            mark = _marker(field_item, node_item)
             FMT += "\n" + "\t" * depth
             FMT += f"{mark}{field_item.name}"
             FMT += "="
             FMT += f"{(_format_node_repr(node_item,depth))}"
-
             FMT += "" if is_last_field else ","
-
-        recurse(node_item, depth, is_frozen)
+        recurse(node_item, depth)
 
     @recurse_field.register(src.tree_base._treeBase)
-    def _(field_item, node_item, depth, is_frozen, is_last_field):
+    def _(field_item: Field, node_item: Any, depth: int, is_last_field: bool):
         """format treeclass field"""
         nonlocal FMT
 
         if field_item.repr:
-            is_frozen = is_treeclass_frozen(node_item)
-            is_static = field_item.metadata.get("static", False)
-            mark = "*" if is_static else ("#" if is_frozen else "")
-
+            mark = _marker(field_item, node_item)
             FMT += "\n" + "\t" * depth
             layer_class_name = f"{node_item.__class__.__name__}"
-
             FMT += f"{mark}{field_item.name}"
             FMT += f"={layer_class_name}" + "("
             start_cursor = len(FMT)  # capture children repr
+            recurse(tree=node_item, depth=depth + 1)
+            temp = FMT[:start_cursor]
+            temp += _format_width(FMT[start_cursor:] + "\n" + "\t" * (depth) + ")")
+            temp += "" if is_last_field else ","
+            FMT = temp
 
-            recurse(
-                node_item, depth=depth + 1, is_frozen=is_treeclass_frozen(node_item)
-            )
+    def recurse(tree, depth):
+        if not is_treeclass(tree):
+            return
 
-            FMT = FMT[:start_cursor] + _format_width(
-                FMT[start_cursor:] + "\n" + "\t" * (depth) + ")"
-            )
-            FMT += "" if is_last_field else ","
-
-    @dispatch(argnum=0)
-    def recurse(tree, depth, is_frozen):
-        ...
-
-    @recurse.register(src.tree_base._treeBase)
-    def _(tree, depth, is_frozen):
         nonlocal FMT
 
         leaves_count = len(_tree_fields(tree))
         for i, fi in enumerate(_tree_fields(tree).values()):
-
-            # retrieve node item
-            cur_node = getattr(tree, fi.name)
-
             recurse_field(
-                fi,
-                cur_node,
-                depth,
-                is_frozen,
-                True if i == (leaves_count - 1) else False,
+                field_item=fi,
+                node_item=getattr(tree, fi.name),
+                depth=depth,
+                is_last_field=True if i == (leaves_count - 1) else False,
             )
 
     FMT = ""
-    recurse(tree, depth=1, is_frozen=is_treeclass_frozen(tree))
+    recurse(tree=tree, depth=1)
     FMT = f"{(tree.__class__.__name__)}(" + _format_width(FMT + "\n)", width)
 
     return FMT.expandtabs(2)
@@ -106,15 +113,15 @@ def tree_str(tree, width: int = 40) -> str:
         str: indented tree leaves.
     """
 
-    @dispatch(argnum=1)
-    def recurse_field(field_item, node_item, depth, is_frozen, is_last_field):
+    @dispatch(argnum="node_item")
+    def recurse_field(
+        field_item: Field, node_item: Any, depth: int, is_last_field: bool
+    ):
         """format non-treeclass field"""
         nonlocal FMT
 
         if field_item.repr:
-            is_static = field_item.metadata.get("static", False)
-            mark = "*" if is_static else ("#" if is_frozen else "")
-
+            mark = _marker(field_item, node_item)
             FMT += "\n" + "\t" * depth
             FMT += f"{mark}{field_item.name}"
             FMT += "="
@@ -127,17 +134,16 @@ def tree_str(tree, width: int = 40) -> str:
 
             FMT += "" if is_last_field else ","
 
-        recurse(node_item, depth, is_frozen)
+        recurse(node_item, depth)
 
     @recurse_field.register(src.tree_base._treeBase)
-    def _(field_item, node_item, depth, is_frozen, is_last_field):
+    def _(field_item: Field, node_item: Any, depth: int, is_last_field: bool):
         """format treeclass field"""
         nonlocal FMT
 
         if field_item.repr:
-            is_frozen = is_treeclass_frozen(node_item)
-            is_static = field_item.metadata.get("static", False)
-            mark = "*" if is_static else ("#" if is_frozen else "")
+            # mark a module static if all its fields are static
+            mark = _marker(field_item, node_item)
 
             FMT += "\n" + "\t" * depth
             layer_class_name = f"{node_item.__class__.__name__}"
@@ -146,60 +152,49 @@ def tree_str(tree, width: int = 40) -> str:
             FMT += f"={layer_class_name}" + "("
             start_cursor = len(FMT)  # capture children repr
 
-            recurse(
-                node_item, depth=depth + 1, is_frozen=is_treeclass_frozen(node_item)
-            )
+            recurse(node_item, depth=depth + 1)
 
-            FMT = FMT[:start_cursor] + _format_width(
-                FMT[start_cursor:] + "\n" + "\t" * (depth) + ")"
-            )
-            FMT += "" if is_last_field else ","
+            temp = FMT[:start_cursor]
+            temp += _format_width(FMT[start_cursor:] + "\n" + "\t" * (depth) + ")")
+            temp += "" if is_last_field else ","
+            FMT = temp
 
-    @dispatch(argnum=0)
-    def recurse(tree, depth, is_frozen):
-        ...
-
-    @recurse.register(src.tree_base._treeBase)
-    def _(tree, depth, is_frozen):
+    def recurse(tree, depth):
+        if not is_treeclass(tree):
+            return
         nonlocal FMT
-
         leaves_count = len(_tree_fields(tree))
         for i, fi in enumerate(_tree_fields(tree).values()):
-
-            # retrieve node item
-            cur_node = tree.__dict__[fi.name]
-
             recurse_field(
-                fi,
-                cur_node,
-                depth,
-                is_frozen,
-                True if i == (leaves_count - 1) else False,
+                field_item=fi,
+                node_item=getattr(tree, fi.name),
+                depth=depth,
+                is_last_field=True if i == (leaves_count - 1) else False,
             )
 
     FMT = ""
-    recurse(tree, depth=1, is_frozen=is_treeclass_frozen(tree))
+    recurse(tree, depth=1)
     FMT = f"{(tree.__class__.__name__)}(" + _format_width(FMT + "\n)", width)
 
     return FMT.expandtabs(2)
 
 
-def _tree_diagram(tree):
-    """
-    === Explanation
-        pretty print treeclass tree with tree structure diagram
+def _tree_diagram(tree: PyTree) -> str:
+    """Pretty print treeclass tree with tree structure diagram
 
-    === Args
-        tree : boolean to create tree-structure
+    Args:
+        tree (PyTree): treeclass tree
+
+    Returns:
+        str: string of tree diagram
     """
 
-    @dispatch(argnum="node_item")
-    def recurse_field(field_item, node_item, is_frozen, parent_level_count, node_index):
+    @dispatch(argnum=1)
+    def recurse_field(field_item, node_item, parent_level_count, node_index):
         nonlocal FMT
 
         if field_item.repr:
-            is_static = field_item.metadata.get("static", False)
-            mark = "*" if is_static else ("#" if is_frozen else "─")
+            mark = _marker(field_item, node_item, default="─")
             is_last_field = node_index <= 1
 
             FMT += "\n"
@@ -211,85 +206,79 @@ def _tree_diagram(tree):
             FMT += f"{field_item.name}"
             FMT += f"={_format_node_diagram(node_item)}"
 
-        recurse(node_item, parent_level_count + [1], is_frozen)
+        recurse(node_item, parent_level_count + [1])
 
     @recurse_field.register(list)
     @recurse_field.register(tuple)
-    def _(field_item, node_item, is_frozen, parent_level_count, node_index):
+    def _(field_item, node_item, parent_level_count, node_index):
         nonlocal FMT
 
         if field_item.repr:
             recurse_field(
-                field_item=field_item,
-                node_item=node_item.__class__,
-                is_frozen=is_frozen,
-                parent_level_count=parent_level_count,
-                node_index=node_index,
+                field_item,
+                node_item.__class__,
+                parent_level_count,
+                node_index,
             )
 
             for i, layer in enumerate(node_item):
-                new_field = field()
+                new_field = field(
+                    metadata={
+                        "static": is_nondiff_field(field_item),
+                        "frozen": is_frozen_field(field_item),
+                    }
+                )
+
                 object.__setattr__(new_field, "name", f"{field_item.name}_{i}")
                 object.__setattr__(new_field, "type", type(layer))
 
                 recurse_field(
-                    field_item=new_field,
-                    node_item=layer,
-                    is_frozen=is_frozen,
-                    parent_level_count=parent_level_count + [node_index],
-                    node_index=len(node_item) - i,
+                    new_field,
+                    layer,
+                    parent_level_count + [node_index],
+                    len(node_item) - i,
                 )
 
-        recurse(node_item, parent_level_count, is_frozen)
+        recurse(node_item, parent_level_count)
 
     @recurse_field.register(src.tree_base._treeBase)
-    def _(field_item, node_item, is_frozen, parent_level_count, node_index):
+    def _(field_item, node_item, parent_level_count, node_index):
         nonlocal FMT
 
         if field_item.repr:
-            is_frozen = is_treeclass_frozen(node_item)
-            is_static = field_item.metadata.get("static", False)
-            mark = "*" if is_static else ("#" if is_frozen else "─")
+            mark = _marker(field_item, node_item, default="─")
             layer_class_name = node_item.__class__.__name__
-
             is_last_field = node_index == 1
-
             FMT += "\n" + "".join(
                 [(("│" if lvl > 1 else "") + "\t") for lvl in parent_level_count]
             )
-
             FMT += f"└{mark}─ " if is_last_field else f"├{mark}─ "
             FMT += f"{field_item.name}"
             FMT += f"={layer_class_name}"
 
-            recurse(node_item, parent_level_count + [node_index], is_frozen)
+            recurse(node_item, parent_level_count + [node_index])
 
-    @dispatch(argnum=0)
-    def recurse(tree, parent_level_count, is_frozen):
-        ...
+    def recurse(tree, parent_level_count):
+        if not is_treeclass(tree):
+            return
 
-    @recurse.register(src.tree_base._treeBase)
-    def _(tree, parent_level_count, is_frozen):
         nonlocal FMT
 
         leaves_count = len(_tree_fields(tree))
 
         for i, fi in enumerate(_tree_fields(tree).values()):
-            cur_node = tree.__dict__[fi.name]
-
             recurse_field(
-                field_item=fi,
-                node_item=cur_node,
-                is_frozen=is_frozen,
-                parent_level_count=parent_level_count,
-                node_index=leaves_count - i,
+                fi,
+                getattr(tree, fi.name),
+                parent_level_count,
+                leaves_count - i,
             )
 
         FMT += "\t"
 
     FMT = f"{(tree.__class__.__name__)}"
 
-    recurse(tree, [1], is_treeclass_frozen(tree))
+    recurse(tree, [1])
 
     return FMT.expandtabs(4)
 
