@@ -39,7 +39,7 @@ def test_freezing_unfreezing():
         c: int = 3
         d: A = A(1, 2)
 
-    @pytc.treeclass(field_only=True)
+    @pytc.treeclass
     class A:
         a: int
         b: int
@@ -94,7 +94,7 @@ def test_freezing_errors():
     class T:
         pass
 
-    @pytc.treeclass(field_only=False)
+    @pytc.treeclass
     class Test:
         a: Any
 
@@ -121,22 +121,11 @@ def test_freezing_with_ops():
         c: int = 3
         d: A = A(1, 2)
 
-    @pytc.treeclass(field_only=False)
+    @pytc.treeclass
     class Test:
         a: int = 1
-        b: float = pytc.static_field(default=(1.0))
-        c: str = "test"
-
-    t = Test()
-
-    with pytest.raises(ImmutableInstanceError):
-        tree_freeze(t).a = 1
-
-    @pytc.treeclass(field_only=True)
-    class Test:
-        a: int = 1
-        b: float = pytc.static_field(default=(1.0))
-        c: str = pytc.static_field(default=("test"))
+        b: float = pytc.nondiff_field(default=(1.0))
+        c: str = pytc.nondiff_field(default=("test"))
 
     t = Test()
     assert jtu.tree_leaves(t) == [1]
@@ -185,6 +174,12 @@ def test_freezing_with_ops():
     t = t1()
     assert is_treeclass_equal(tree_unfreeze(tree_freeze(t)), t)
 
+    @pytc.treeclass
+    class t2:
+        a: int = t1()
+
+    assert is_treeclass_equal(tree_unfreeze(tree_freeze(t2())), t2())
+
 
 def test_freeze_diagram():
     @pytc.treeclass
@@ -214,3 +209,18 @@ def test_freeze_diagram():
     assert (
         tree_diagram(a)
     ) == "B\n    ├── c=3\n    └#─ d=A\n        ├#─ a=1\n        └#─ b=2     "
+
+
+def test_freeze_mask():
+    @pytc.treeclass
+    class Test:
+        a: int = 1
+        b: int = 2
+        c: float = 3.0
+
+    t = Test()
+
+    assert jtu.tree_leaves(tree_freeze(t, t == "a")) == [2, 3.0]
+    assert jtu.tree_leaves(tree_freeze(t, t == "b")) == [1, 3.0]
+    assert jtu.tree_leaves(tree_freeze(t, t == "c")) == [1, 2]
+    assert jtu.tree_leaves(tree_freeze(t, t == t)) == []
