@@ -21,7 +21,7 @@ class Test:
     b: float
     c: float
     d: jnp.ndarray
-    name: str = pytc.static_field()
+    name: str = pytc.nondiff_field()
 
 
 def test_getter_by_val():
@@ -132,7 +132,7 @@ def test_getter_by_metadata():
         b: float = field(metadata={"name": "b"})
         c: float = field(metadata={"name": "c"})
         d: jnp.ndarray = field(metadata={"name": "d"})
-        name: str = pytc.static_field()
+        name: str = pytc.nondiff_field()
 
     A = Test(10, 20, 30, jnp.array([1, 2, 3, 4, 5]), ("A"))
 
@@ -503,7 +503,7 @@ def test_apply_and_its_derivatives():
         b: float = field(metadata={"name": "b"})
         c: float = field(metadata={"name": "c"})
         d: jnp.ndarray = field(metadata={"name": "d"})
-        name: str = pytc.static_field()
+        name: str = pytc.nondiff_field()
 
     A = Test(10, 20, 30, jnp.array([1, 2, 3, 4, 5]), ("A"))
 
@@ -563,7 +563,7 @@ def test_apply_and_its_derivatives():
 
     t = L2()
     lhs = t.at[t != {"name": "a"}].apply(lambda _: 100)
-    rhs = L2(10, 100, 100, L1(1, 100, 100, L0(1, 100, 100)))
+    rhs = L2(10, 100, 100, L1(100, 100, 100, L0(100, 100, 100)))
 
     assert is_treeclass_equal(lhs, rhs)
 
@@ -953,3 +953,29 @@ def test_repr_str():
 
     assert f"{t.at['a']!r}" == "where='a'"
     assert f"{t.at['a']!s}" == "where=a"
+
+
+def test_not_equal():
+    @pytc.treeclass
+    class Test:
+        a: int = 1
+        b: float = 1.0
+
+    t = Test()
+
+    assert is_treeclass_equal(t.at[t == int].set(10), Test(a=10, b=1.0))
+
+    assert is_treeclass_equal(t.at[t != int].set(10.0), Test(a=1, b=10.0))
+
+    assert is_treeclass_equal(t.at[t != 10].set(10.0), Test(a=10.0, b=10.0))
+
+
+def test_iterable_node():
+    @pytc.treeclass
+    class Test:
+        a: int
+
+    t = Test([1, 2, 3, 4])
+    assert is_treeclass_equal(t.at[...].set(True), Test([True, True, True, True]))
+    assert is_treeclass_equal(t.at[t == "a"].set(True), Test([True, True, True, True]))
+    assert is_treeclass_equal(t.at[t != "a"].set(True), Test([1, 2, 3, 4]))
